int main(void)
{
  int arr[] = {1, 2, 3, 4, 5};
  arr[2] = 99;
  int arr_0_after_write = arr[0];
  int arr_1_after_write = arr[1];
  int arr_2_after_write = arr[2];
  int arr_3_after_write = arr[3];
  int arr_4_after_write = arr[4];
  return 0;
}
