public class Test3
{
    public static void main(String input_string)
    {
        int parsed1 = Integer.parseInt(input_string);
        if (parsed1 == 2) {
            assert(false);
        }
    }
}
