
class ClassWithFields {
  final boolean final1 = true;
  final Boolean final2 = false;
  final Object final3 = null;

  boolean nonFinal1 = true;
  Boolean nonFinal2 = false;
  Object nonFinal3 = null;
}
