public class Test {
  public static void test(boolean b) {
    int x;
    if (b) {
      x = ClassWithStaticInit.getStaticValue();
    }
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    x = ClassWithStaticInit.getStaticValue();
    assert (false);
  }
}
