package org.cprover.first;

import org.cprover.second.RelatedClassOtherPkg;

public class RelevantClass {

  public static void entry(RelatedClass rc, RelatedClassOtherPkg rcop) {
    assert false;
  }
}
