public class StringConcatenation02
{
   public static void main(String[] args)
   {
      String s1 = "Happy at";
      String s2 = " Diffblue";
      assert s1.equals("Happy at ");
      assert s2.equals(" Diffblue");
   }
}
