#include <assert.h>

void foo()
{
  static int local;
  local = 1;
}

int main()
{
  // foo::1::local is the fully qualified name generated by the C front-end at
  // this time. This is an implementation detail that could change at any time,
  // and the test would need to be adjusted accordingly.
  assert(__CPROVER_ID "foo::1::local" == 0);
  foo();
  assert(__CPROVER_ID "foo::1::local" == 1);
}
