public class StringIndexMethods05
{
   public static void main(String[] args)
   {
      String letters = "automatictestcasegenerationatdiffblue";
      assert letters.lastIndexOf("diffblue", 25)==1;
   }
}
