public class test_last_index_of2
{
   public static void main(String[] args)
   {
      String letters = "automatictestcasegenerationatdiffblue";
      assert(letters.lastIndexOf("diffblue", 25)==-1);
   }
}
