// None of these types should be defined when emulating gcc-4, which clang on
// macOS also emulates:

typedef float _Float32;
typedef double _Float32x;
typedef double _Float64;
typedef long double _Float64x;
typedef long double _Float128;
typedef long double _Float128x;
