public class Main {
  public void test1() {
    String s1 = "abc";
    s1.substring(-1);
  }

  public void test2() {
    String s1 = "abc";
    s1.substring(4);
  }

  public void test3() {
    String s1 = "abc";
    s1.substring(-1, 4);
  }
}
