// Author: Pascal Kesseli, pascal.kesseli@diffblue.com

namespace asdf {}

namespace
asdf
{}

namespace
asdf
  {}

  namespace


asdf
{}

namespace xyz = my::nested::namespaces;



namespace {}

namespace
{ }

namespace
{
}

    namespace
{
}
