/*******************************************************************\

Module: Static initializer call lifting

Author: Diffblue Ltd.

\*******************************************************************/

/// file Static initializer call lifting

#ifndef CPROVER_JAVA_BYTECODE_LIFT_CLINIT_CALLS_H
#define CPROVER_JAVA_BYTECODE_LIFT_CLINIT_CALLS_H

#include <util/std_code_base.h>

codet lift_clinit_calls(codet input);

#endif // CPROVER_JAVA_BYTECODE_LIFT_CLINIT_CALLS_H
