int main(int argc, char argv[])
{
  int unknown;
  int p;

  switch(unknown)
  {
  case 0:
    p = 0;
    break;
  case 1:
    p = 1;
    break;
  default:
    p = -1;
    break;
  }

  int q = !p;
}
