class Basic1
{
    public static void main(String[] args) {
        String s = "Hello ";
        s +=  "World!";
        if(s.equals("Hello World!"))
            System.out.printf("Hello World!");
    }
};

